import 'package:dartlin/control_flow.dart';
import 'package:dartlin/ranges.dart' show range;
import 'package:flutter_test/flutter_test.dart' show test, expect;

void main() {
  test('Should return the right value in the simplest form', () {
    String testWhen(int value) {
      return when(value, {
        1: () => '1',
        2: () => '2',
        orElse: () => 'else',
      });
    }

    expect(testWhen(1), '1');
    expect(testWhen(2), '2');
    expect(testWhen(3), 'else');
  });

  test('Should return the right value when given a list of possible options',
      () {
    String testWhen(int value) {
      return when<dynamic, String>(value, {
        [1, 2]: () => '1',
        2: () => '3', // Should never reach.
        orElse: () => 'else',
      });
    }

    expect(testWhen(1), '1');
    expect(testWhen(2), '1');
    expect(testWhen(3), 'else');
  });

  test('Should handle when check methods correctly', () {
    String testWhen(int value) {
      return when(value, {
        isIn(range(0, to: 10)): () => '1',
        isNotIn(range(10, to: 20)): () => '2',
        orElse: () => 'else',
      });
    }

    expect(testWhen(1), '1');
    expect(testWhen(2), '1');
    expect(testWhen(22), '2');
    expect(testWhen(11), 'else');
  });

  test('Should be able to handle simple bool values', () {
    String testWhen(int value) {
      return when({
        value.isOdd: () => '1',
        value.isEven: () => '2',
      });
    }

    expect(testWhen(1), '1');
    expect(testWhen(2), '2');
  });
}
